@@ -5,8 +5,10 @@ package themes
 import (
 	"fmt"
 	"io"
+	"maps"
 	"path/filepath"
 	"regexp"
+	"slices"
 	"strings"
 	"time"
 
@@ -16,9 +18,6 @@ import (
 	"kitty/tools/tui/readline"
 	"kitty/tools/utils"
 	"kitty/tools/wcswidth"
-
-	"golang.org/x/exp/maps"
-	"golang.org/x/exp/slices"
 )
 
 var _ = fmt.Print
@@ -327,6 +326,11 @@ func (self *handler) draw_browsing_screen() {
 		}
 		self.lp.MoveCursorHorizontally(mw - l.width)
 		self.lp.Println(SEPARATOR)
+		num_rows--
+	}
+	for ; num_rows > 0; num_rows-- {
+		self.lp.MoveCursorHorizontally(mw + 1)
+		self.lp.Println(SEPARATOR)
 	}
 	if self.themes_list != nil && self.themes_list.Len() > 0 {
 		self.draw_theme_demo()
@@ -447,10 +451,11 @@ func (self *handler) draw_theme_demo() {
 				if intense {
 					s = "bright-" + s
 				}
-				if len(s) > trunc {
-					s = s[:trunc]
+				sTrunc := s
+				if len(sTrunc) > trunc {
+					sTrunc = sTrunc[:trunc]
 				}
-				buf.WriteString(self.lp.SprintStyled("fg="+s, s))
+				buf.WriteString(self.lp.SprintStyled("fg="+s, sTrunc))
 				buf.WriteString(" ")
 			}
 			text := strings.TrimSpace(buf.String())
