@@ -5,10 +5,8 @@ import os
 import sys
 import tempfile
 
-from kitty.config import build_ansi_color_table, defaults
 from kitty.fast_data_types import (
     Color,
-    ColorProfile,
     HistoryBuf,
     LineBuf,
     expand_ansi_c_escapes,
@@ -476,14 +474,6 @@ class TestDataTypes(BaseTest):
         self.ae(sanitize_url_for_dispay_to_user(
             'h://a\u0430b.com/El%20Ni%C3%B1o/'), 'h://xn--ab-7kc.com/El Niño/')
 
-    def test_color_profile(self):
-        c = ColorProfile()
-        c.update_ansi_color_table(build_ansi_color_table())
-        for i in range(8):
-            col = getattr(defaults, f'color{i}')
-            self.ae(c.as_color(i << 8 | 1), col)
-        self.ae(c.as_color(255 << 8 | 1), Color(0xee, 0xee, 0xee))
-
     def test_historybuf(self):
         lb = filled_line_buf()
         hb = HistoryBuf(5, 5)
@@ -594,7 +584,7 @@ class TestDataTypes(BaseTest):
         self.assertNotEqual(SingleKey(key=1, mods=2), SingleKey(key=1))
 
     def test_notify_identifier_sanitization(self):
-        from kitty.notify import sanitize_identifier_pat
+        from kitty.notifications import sanitize_identifier_pat
         self.ae(sanitize_identifier_pat().sub('', '\x1b\nabc\n[*'), 'abc')
 
     def test_bracketed_paste_sanitizer(self):
@@ -602,7 +592,7 @@ class TestDataTypes(BaseTest):
         for x in ('\x1b[201~ab\x9b201~cd', '\x1b[201\x1b[201~~ab'):  # ]]]
             q = sanitize_for_bracketed_paste(x.encode('utf-8'))
             self.assertNotIn(b'\x1b[201~', q)
-            self.assertNotIn('\x9b201~'.encode('utf-8'), q)
+            self.assertNotIn('\x9b201~'.encode(), q)
             self.assertIn(b'ab', q)
 
     def test_expand_ansi_c_escapes(self):
@@ -639,7 +629,7 @@ class TestDataTypes(BaseTest):
             r'x "ab"y \m': ((0, 'x'), (2, 'aby'), (8, 'm')),
             r'''x'y"\z'1''': ((0, 'xy"\\z1'),),
             r'\abc\ d': ((0, 'abc d'),),
-            '': (), '   ': (), ' \tabc\n\t\r ': ((2, 'abc'),),
+            '': ((0, ''),), '   ': ((0, ''),), ' \tabc\n\t\r ': ((2, 'abc'),),
             "$'ab'": ((0, '$ab'),),
         }.items():
             actual = tuple(shlex_split_with_positions(q))
